/*
   SPDX-FileCopyrightText: 2020 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>

   SPDX-License-Identifier: LGPL-3.0-or-later
 */

#include "elisaarguments.h"

ElisaArguments::ElisaArguments(QObject *parent)
    : QObject(parent)
{
}


#include "moc_elisaarguments.cpp"
